#include "input_modplay.h"

enum
{
   raw_bits_per_sample = 16,
   raw_channels = 2,
   raw_sample_rate = 44100,

   raw_bytes_per_sample = raw_bits_per_sample / 8,
   raw_total_sample_width = raw_bytes_per_sample * raw_channels,
};


// Note that input class does *not* implement virtual methods or derive from interface classes.
// Our methods get called over input framework templates. See input_singletrack_impl for descriptions of what each method does.
// input_stubs just provides stub implementations of mundane methods that are irrelevant for most implementations.
void input_modplay::open( service_ptr_t<file> p_filehint, const char *p_path, t_input_open_reason p_reason, abort_callback &p_abort )
{
   MODFILE_Init( &m_mod );
   pfc::array_t<t_uint8> buffer;
   service_ptr_t<file> file;
   input_open_file_helper( file, p_path, p_reason, p_abort );
   m_filestats = file->get_stats( p_abort );
   buffer.set_size( file->get_size( p_abort ) );
   size_t got = file->read( buffer.get_ptr(), buffer.get_size(), p_abort );
	int result = MODFILE_Set( buffer.get_ptr(), buffer.get_size(), &m_mod );
   m_mod.musicvolume = 64;
   MODFILE_Start( &m_mod );
   MODFILE_SetFormat( &m_mod, raw_sample_rate, raw_channels, raw_bits_per_sample, 1 );
}


void input_modplay::get_info( file_info &p_info, abort_callback &p_abort )
{
	//note that the values below should be based on contents of the file itself, NOT on user-configurable variables for an example. To report info that changes independently from file contents, use get_dynamic_info/get_dynamic_info_track instead.
	p_info.info_set_int( "samplerate", raw_sample_rate );
	p_info.info_set_int( "channels", raw_channels );
	p_info.info_set_int( "bitspersample", raw_bits_per_sample );

	// Indicate whether this is a fixedpoint or floatingpoint stream, when using bps >= 32
	// As 32bit fixedpoint can't be decoded losslessly by fb2k, does not fit in float32 audio_sample.
	if ( raw_bits_per_sample >= 32 )
      p_info.info_set( "bitspersample_extra", "fixed-point" );

	p_info.info_set( "encoding", "lossless" );
	p_info.info_set_bitrate( ( raw_bits_per_sample * raw_channels * raw_sample_rate + 500 /* rounding for bps to kbps*/ ) / 1000 /* bps to kbps */ );
		
}


t_filestats input_modplay::get_file_stats( abort_callback &p_abort )
{
   return( m_filestats );
}


void input_modplay::decode_initialize( unsigned p_flags,abort_callback &p_abort )
{
}


bool input_modplay::decode_run(audio_chunk & p_chunk,abort_callback & p_abort)
{
	enum
   {
		deltaread = 1024
	};

   m_buffer.set_size( deltaread * 4 );
   u16 *pBuf = (u16*)m_buffer.get_ptr();
   m_mod.mixingbuf = pBuf;
   m_mod.mixingbuflen = deltaread * 4;
   MODFILE_Player( &m_mod );

   p_chunk.set_data_fixedpoint( pBuf, deltaread * 4, raw_sample_rate, raw_channels, raw_bits_per_sample, audio_chunk::g_guess_channel_config( raw_channels ) );

   return( true );
}


void input_modplay::decode_seek(double p_seconds,abort_callback & p_abort)
{
}


bool input_modplay::decode_can_seek()
{
      return( false );
}


bool input_modplay::decode_get_dynamic_info( file_info &p_out, double &p_timestamp_delta )
{
   return( false );
}


bool input_modplay::decode_get_dynamic_info_track( file_info &p_out, double &p_timestamp_delta )
{
   return( false );
}


void input_modplay::decode_on_idle( abort_callback &p_abort )
{
   /* Nothing to be done */
}


void input_modplay::retag( const file_info &p_info, abort_callback &p_abort)
{
   throw exception_io_unsupported_format();
}


bool input_modplay::g_is_our_content_type( const char * p_content_type )
{
   return( false );
}


bool input_modplay::g_is_our_path( const char * p_path,const char * p_extension )
{
	return( stricmp_utf8( p_extension, "mod" ) == 0 ||
           stricmp_utf8( p_extension, "s3m" ) == 0 ||
           stricmp_utf8( p_extension, "xm" ) == 0 );
}


const char *input_modplay::g_get_name()
{
   return "foo_modplay MOD input";
}


const GUID input_modplay::g_get_guid()
{
   // GUID of the decoder. Replace with your own when reusing code.
   static const GUID I_am_foo_sample_and_this_is_my_decoder_GUID = { 0xd9c01c8d, 0x69c5, 0x4eec,{ 0xa2, 0x1c, 0x1d, 0x14, 0xef, 0x65, 0xbf, 0x8b } };
   return I_am_foo_sample_and_this_is_my_decoder_GUID;
}


static input_singletrack_factory_t<input_modplay> g_input_modplay_factory;

// Declare .RAW as a supported file type to make it show in "open file" dialog etc.
DECLARE_FILE_TYPE( "MOD Files", "*.mod;*.s3m;*.xm" );
